import TypeUtils from "./utils/type"

import ActionsFormsSubmitV1 from "./actions/forms/submit"
import ActionsFormsEncodeV1 from "./actions/forms/encode"

import ActionsHttpPostV1 from "./actions/http/post"
import ActionsHttpPatchV1 from "./actions/http/patch"
import ActionsHttpDeleteV1 from "./actions/http/delete"

import ActionsDialogsAlert from "./actions/dialogs/alert"
// import ActionsDialogsConfirm from "./actions/dialogs/confirm"
import ActionsDialogsShow from "./actions/dialogs/show"
import ActionsDialogsOpen from "./actions/dialogs/open"
import ActionsDialogsClose from "./actions/dialogs/close"
import ActionsDialogsOptions from "./actions/dialogs/options"
import ActionsDialogsNotification from "./actions/dialogs/notification"
import ActionsDialogsOauth from "./actions/dialogs/oauth"

import ActionsSnackbarsAlert from "./actions/snackbars/alert"
// import ActionsSnackbarsConfirm from "./actions/snackbars/confirm"
import ActionsSnackbarsSelect from "./actions/snackbars/select"

import ActionsSheetsSelect from "./actions/sheets/select"

import ActionsWindowsClose from "./actions/windows/close"
import ActionsWindowsCloseAll from "./actions/windows/closeAll"
import ActionsWindowsOpen from "./actions/windows/open"
import ActionsWindowsOpenWeb from "./actions/windows/openWeb"
import ActionsWindowsReload from "./actions/windows/reload"

import ActionsAuthSaveCsrfToken from "./actions/auth/saveCsrfToken"
import ActionsAuthRestart from "./actions/auth/restart"

const actions = {
  "forms/submit": ActionsFormsSubmitV1,
  "forms/encode": ActionsFormsEncodeV1,

  "http/post": ActionsHttpPostV1,
  "http/patch": ActionsHttpPatchV1,
  "http/delete": ActionsHttpDeleteV1,

  "dialogs/alert": ActionsDialogsAlert,
  // "dialogs/confirm": ActionsDialogsConfirm,
  "dialogs/show": ActionsDialogsShow,
  "dialogs/open": ActionsDialogsOpen,
  "dialogs/close": ActionsDialogsClose,
  "dialogs/options": ActionsDialogsOptions,
  "dialogs/notification": ActionsDialogsNotification,
  "dialogs/oauth": ActionsDialogsOauth,

  "snackbars/alert": ActionsSnackbarsAlert,
  // "snackbars/confirm": ActionsSnackbarsConfirm,
  "snackbars/select": ActionsSnackbarsSelect,

  "sheets/select": ActionsSheetsSelect,

  "windows/close": ActionsWindowsClose,
  "windows/closeAll": ActionsWindowsCloseAll,
  "windows/open": ActionsWindowsOpen,
  "windows/openWeb": ActionsWindowsOpenWeb,
  "windows/reload": ActionsWindowsReload,

  "auth/saveCsrfToken": ActionsAuthSaveCsrfToken,
  "auth/restart": ActionsAuthRestart,
};

export default class Action {
  static execute(spec, target, component, params = {}) {
    if (!TypeUtils.isObject(spec)) {
      return
    }

    const name = spec["action"]
    if (TypeUtils.isString(name)) {
      const actionName = name.replace(/-v1$/, '')

      console.log(`Executing "${actionName}"`)

      try {
        const action = new actions[actionName]()
        action.execute(spec, target, component, params)
      }
      catch (e) {
        console.log(`Failed executing command: "${e.message}"`)
      }
    }
    else {
      console.log(`Invalid action: "${name}"`)
    }
  }
}

window.Action = Action
