import Action from "../../action";
import UrlUtils from "../../utils/url";
import TypeUtils from "../../utils/type";

export default {
  data: function() {
    return {
      // TODO: Consider replacing with an enum, e.g. executing with predefined values ('post', 'get', null)
      // The complexity with this is that 'get' that comes after 'post' needs to show an indicator, e.g.
      // Submit button needs to be disabled when redirecting after form submission
      _isBusy: false,
      _events: []
    };
  },
  computed: {
    _pageBody() {
      return document.getElementById("page_body");
    },
    $isBusy() {
      // Properties starting with $ or _ can only be accessed via $data
      return this.$data._isBusy;
    },
    $isReady() {
      return this.$data._events === null;
    }
  },
  mounted: function() {
    this._executeIfReady(this._ready, false);
  },
  updated: function() {
    this._executeIfReady(this._ready, true);
  },
  watch: {
    // See Utils.http
    "$root.isStale": function(val, oldVal) {
      // Make sure $ready() will be called in the next update
      this._mountedUrl = null;
    }
  },
  methods: {
    $href: function(spec) {
      const properties = spec || this.spec;
      const onClick = properties.onClick;
      if (TypeUtils.isObject(onClick)) {
        switch (onClick.action) {
          case "windows/open-v1":
          case "windows/openWeb-v1":
          case "windows/reload-v1":
            return UrlUtils.htmlUrl(onClick.url);
        }
      }
      return null;
    },
    $onClick: function(e, spec) {
      const properties = spec || this.spec;

      // Ignore middle/right clicks
      if (!event.metaKey && !event.ctrlKey && event.button === 0) {
        event.preventDefault();
        event.stopPropagation();
        Action.execute(properties.onClick, event.target, this);
      }
    },
    $rel: function(spec) {
      const properties = spec || this.spec;
      const onClick = properties.onClick;
      if (TypeUtils.isObject(onClick)) {
        if (onClick.action === "windows/openWeb-v1") {
          return "nofollow";
        }
      }
      return null;
    },
    $length: function(length) {
      if (TypeUtils.isString(length) || TypeUtils.isNumber(length)) {
        switch (length) {
          case "wrapContent":
            return null;
          case "matchParent":
            return "100%";
          default:
            return `${length}px`;
        }
      }
      return null;
    },
    $builtinColor(styleClasses) {
      let color = null
      Utils.type.ifArray(styleClasses, classes => {
        for (const val of ["success", "info", "warning", "error"]) {
          if (classes.remove(val)) {
            color = val
          }
        }
      })
      return color
    },
    $addViewportChangeListeners: function(handler) {
      window.addEventListener("load", handler);
      window.addEventListener("resize", handler);
      this._pageBody.addEventListener("scroll", handler);
    },
    $removeViewportChangeListeners: function(handler) {
      window.removeEventListener("load", handler);
      window.removeEventListener("resize", handler);
      this._pageBody.removeEventListener("scroll", handler);
    },
    // To be used in `mounted()` and `updated()`
    _executeIfReady(handler, updated) {
      if (updated) {
        if (!this._renderingTheSamePage()) {
          handler();
        }
      } else {
        handler();
      }
    },
    _renderingTheSamePage() {
      return this._mountedUrl === window.location.href;
    },
    _ready() {
      this._mountedUrl = window.location.href;
      this._updatedAt = new Date().getTime();

      this.$data._events = [];

      // Has to be executed before $ready()
      this._linkFieldModels();

      this.$ready();

      // Make sure events are dispatched after $ready(). Dispatching them during $ready() causes errors.
      for (const event of this.$data._events) {
        // Execute later to ensure the parent has registered its event listener
        // setTimeout(() => {
        this.$el.dispatchEvent(event);
        // }, 100)
      }
      this.$data._events = null;
    },
    $ready() {
      // To be overridden
    },
    $dispatchEvent(name, data) {
      const event = new Event(name, { bubbles: true });

      // TODO: Deprecate
      event.spec = data;

      event.data = data;

      if (this.$data._events) {
        this.$data._events.push(event);
      } else {
        this.$el.dispatchEvent(event);
      }
    },
    $onEvent(name, handler) {
      this.$el.addEventListener(name, handler);
    },
    $closest(name) {
      var parent = this.$parent
      while (parent != null) {
        if (Utils.type.isObject(parent.spec) && Utils.app.vueName(parent) == name) {
          return parent
        }
        parent = parent.$parent
     }
    }
  }
};
