import Hash from "../../utils/hash";
import Vue from 'vue'

var jsonLogic = require("json-logic-js");

export default {
  data: function() {
    return {
      _fieldModels: {},

      // To be set by the implementation
      fieldName: null,
      fieldModel: null
    };
  },
  computed: {
    display() {
      const conditions = this.spec.showIf
      if (conditions) {
        return jsonLogic.apply(conditions, this.$data._fieldModels) ? "block" : "none"
      } else {
        return "block"
      }
    }
  },
  watch: {
    fieldModel: function(val, oldVal) {
      // Make the field property observable for dynamic showing/hiding
      Vue.set(this.$data._fieldModels, this.fieldName, val)
    }
  },
  methods: {
    // TODO: Deprecated
    genericStyles(spec) {
      return this.$styles(spec)
    },
    // NOTE: Styles are dynamic, do not save it in $ready().
    $styles(spec) {
      const properties = spec || this.spec;
      const styles = {
        color: properties.color,
        "background-color": properties.backgroundColor,
        "text-align": properties.textAlign
      };

      const padding = properties.padding;
      if (Utils.type.isObject(padding)) {
        Utils.type.ifNumber(
          padding.top,
          top => (styles["padding-top"] = `${top}px`)
        );
        Utils.type.ifNumber(
          padding.bottom,
          bottom => (styles["padding-bottom"] = `${bottom}px`)
        );
        Utils.type.ifNumber(
          padding.left,
          left => (styles["padding-left"] = `${left}px`)
        );
        Utils.type.ifNumber(
          padding.right,
          right => (styles["padding-right"] = `${right}px`)
        );
      }

      Utils.type.ifString(
        this.$length(properties.width),
        val => (styles["width"] = val)
      );
      Utils.type.ifString(
        this.$length(properties.height),
        val => (styles["height"] = val)
      );

      Utils.type.ifObject(
        properties.onClick,
        val => (styles["cursor"] = "pointer")
      );

      // This is the main reason styles have to be used in a dynamic way.
      this._updateDisplay(styles, properties);

      return new Hash(styles);
    },
    _updateDisplay(styles, properties) {
      Utils.type.ifObject(
        properties.showIf,
        val => (styles["display"] = this.display)
      );
    },
    _linkFieldModels() {
      const hasCondition = this.spec && this.spec.showIf
      const name = Utils.app.vueName(this)
      const isField = name && name.startsWith('fields-')
      if (hasCondition || isField) {
        const form = this.$closest('panels-form')
        if (form != null) {
          this.$data._fieldModels = form.$data._fieldModels

          // Has to be executed before $ready()
          this.fieldName = this.spec.name
          this.fieldModel = this.spec.value
        }
      }
    },
    $classes(spec) {
      const properties = spec || this.spec;
      const componentName = Utils.app.componentName(properties.view);
      const classes = (properties.styleClasses || []).concat(componentName);

      Utils.type.ifArray(properties.styleClasses, val => {
        if (val.includes("card")) {
          classes.push("v-card");
          classes.push("v-sheet");
          classes.push("theme--light");

          if (val.includes("outlined")) {
            classes.push("v-card--outlined");
          }
        }
      });
      return classes;
    },
  }
};
